package jalview.gui;

import jalview.jbgui.GAlignmentPanel;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

public class AlignmentPanel extends GAlignmentPanel implements AdjustmentListener
{

  AlignViewport     av;
  public SeqPanel   seqPanel;
  public IdPanel    idPanel;
  public AlignFrame alignFrame;
  public ScalePanel scalePanel;
  public ScorePanel scorePanel;

  public boolean    groupEdit = false;

  public AlignmentPanel(AlignFrame af, AlignViewport av)
  {
    alignFrame = af;
    this.av         = av;
    seqPanel        = new SeqPanel  (av, this);
    idPanel         = new IdPanel   (av, this);
    scalePanel = new ScalePanel(av);
    scorePanel = new ScorePanel(av);

    idPanelHolder.add(idPanel, BorderLayout.CENTER);
    scalePanelHolder.add(scalePanel, BorderLayout.CENTER);
    scorePanelHolder.add(scorePanel, BorderLayout.CENTER);
    seqPanelHolder.add(seqPanel, BorderLayout.CENTER);
    setScrollValues(0,0);

    hscroll.addAdjustmentListener(this);
    vscroll.addAdjustmentListener(this);
    selectAll(false);

    addComponentListener(new ComponentAdapter()
   {
       public void componentResized(ComponentEvent evt)
     {      RefreshPanels();    }
   });

  }

  public int countSelected()
  {
    return av.getSelection().size();
  }

  public void selectAll(boolean flag) {
    int i = 0;
    if (flag) {
      while (i < seqPanel.align.getHeight()) {
        if (! av.getSelection().contains(av.getAlignment().getSequenceAt(i))) {
          av.getSelection().addElement(av.getAlignment().getSequenceAt(i));
        }
        i++;
      }
    } else {
      av.getSelection().clear();
    }
  }



  public void setGraphPanelVisible(boolean b)
  {
    scorePanelHolder.setVisible(b);
    idSpaceFillerPanel.setVisible(b);
    RefreshPanels();
  }



  public void RefreshPanels()
  {
    invalidate();
    idPanelHolder.setPreferredSize( idPanel.idCanvas.getPreferredSize() );
    validate();
    idPanel.idCanvas.paintFlag = true;
    seqPanel.seqCanvas.paintFlag = true;
    setScrollValues(av.getStartRes(), av.getStartSeq());
    av.getConsensus(true);
    repaint();
  }

  int hextent = 0;
  int vextent = 0;

  public void setScrollValues(int x, int y)
  {
    hextent = (int)(seqPanel.seqCanvas.getWidth()/av.getCharWidth());
    vextent = seqPanel.seqCanvas.getHeight()/av.getCharHeight();

    if(hextent+x  >  av.getAlignment().getWidth())
      x =  av.getAlignment().getWidth()- hextent;

    if(vextent+y > av.getAlignment().getHeight())
      y = av.getAlignment().getHeight() - vextent;

    if(y<0)
      y = 0;

    if(x<0)
      x=0;

    hscroll.setValues(x,hextent,0,av.getAlignment().getWidth());
    vscroll.setValues(y,vextent,0,av.getAlignment().getHeight() );

  }


  public void adjustmentValueChanged(AdjustmentEvent evt) {

    if (evt.getSource() == hscroll)
    {
      int x = hscroll.getValue();
      av.setStartRes(x);
      av.setEndRes(x + (int)(seqPanel.seqCanvas.getWidth()/av.getCharWidth()-1));
    }

    if (evt.getSource() == vscroll)
    {
        int offy = vscroll.getValue();
        av.setStartSeq(offy);
        av.setEndSeq(offy + seqPanel.seqCanvas.getHeight()/av.getCharHeight());
    }

    repaint();
  }



}
